import {
  Metadata, Loader, IEventParam, Http, Config, IDictionary, Datasource,
} from '@zeedhi/core';
import {
  Select, TreeGridEditable, Form, IComponentRender, ModalService, Modal, GridEditable, DialogService, Button, Tabs,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pla01300PratoController {
  public crudController = Loader.getInstance('CrudController');

  public modalController = Loader.getInstance('ModalController');
  
  private utilities = Loader.getInstance('Utilities');

  public gridEditController = Loader.getInstance('GridEditableController');

  public appController = Loader.getInstance('AppController');
  
  public receita = Loader.getInstance('ReceitaController');

  public modalAddPlate!: Modal;
  
  public modalImport!: Modal;
  
  private recipeDest = '';

  public codeTree: IDictionary = {};

  public levelMax: any = '';

  public plateLocal: string = 'N';

  public setFilterUnidadeDetail({ component }: IEventParam<Select>) {
    const values = {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA'),
      NMFILIAL: localStorage.getItem('NMFILIALLOGADA'),
    };
    component.datasource.data = [values];
    component.setValue(localStorage.getItem('CDFILIALLOGADA'));
  }

  public changePlate({ component }: IEventParam<Datasource>) {
    const cardPratos: any = Metadata.getInstance('cardTabs');
    const tabPratos: Tabs = Metadata.getInstance('tabPratos');
    const maskLevel = this.getLevelByMask(1);
    const nivelMax = maskLevel.length.toString();

    const plate = component.currentRow.CDPRATO;
    const levelPlate = component.currentRow.CDNVPRATO;
    const auxPlate = component.currentRow.CDFILAUXPRAT;
    const unity = component.currentRow.CDFILIAL;
    
    if (plate) {
      if (levelPlate === nivelMax) {
        this.setDataSourcePlateData(plate, auxPlate, unity);
        this.scrollTab();
        this.mountedDadosPlate();
        this.cancelForm();
        this.isVisibleTabTypeMenu();
        this.visibleRecipeStandardQualityAndRegion();
        cardPratos.isVisible = true;
        if (tabPratos.activeTab !== 0 && tabPratos.activeTab === 2) this.setfilterRecipeStandard();
        else if (tabPratos.activeTab !== 0 && tabPratos.activeTab === 3) this.setfilterRecipeTpCard();
        else if (tabPratos.activeTab !== 0) {
          this.setFilterUnidadeLocal();
          this.setFilterUnidadeService();
          this.setfilterRecipeRegion();
          this.setfilterRecipePadraoQualidade();
        }
      } else cardPratos.isVisible = false;
    } else cardPratos.isVisible = false;
  }
  
  public closeTabs() {
    const tabPratos: any = Metadata.getInstance('cardTabs');
    tabPratos.isVisible = false;
  }

  public scrollTab() {
    const element = document.getElementsByName('tabPratos')[0];

    setTimeout(() => {
      element.scrollIntoView();
    }, 1);
  }

  public setDataSourcePlateData(plate: string, auxPlate: string, unity: string) {
    const gridPlateData = Metadata.getInstance<GridEditable>('gridDadosPrato');
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: plate,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: auxPlate,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unity,
      },
    ];
    
    gridPlateData.datasource.filter = conditions;
    gridPlateData.datasource.get();
  }

  public setFilterBranch({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterPlateTree({ component }: IEventParam<Select>) {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const buttonAdd = Metadata.getInstance<Button>('addTree');
    const buttonSeach = Metadata.getInstance<Button>('searchTreeGridPlate');
    if (component.value !== null) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: component.value.CDFILIAL,
        },
      ];
      
      plateTree.datasource.filter = conditions;
      plateTree.datasource.filter.limit = 1000;
      plateTree.buildTree();
      buttonAdd.disabled = false;
      buttonSeach.disabled = false;
    } else {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: null,
        },
      ];
      plateTree.datasource.filter = conditions;
      plateTree.datasource.filter.limit = 1000;
      plateTree.buildTree();
      buttonAdd.disabled = true;
      buttonSeach.disabled = true;
    }
  }

  public addArvPlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const form = Metadata.getInstance<Form>('formUnidade');
    const row = form.value;

    row.COLUMN_NAME = 'CDGROPERREPA';
    row.ALIAS_NAME = 'ALTRECEPADRAO';
    Http.post('/prato/returnParamsAddPlate', { requestType: 'Row', row }).then((callback) => {
      if (callback && callback.data.dataset && callback.data.dataset.permission && callback.data.dataset.paramgeral.length) {
        const result = callback.data.dataset.permission.result;
        const receitaRevisada = callback.data.dataset.paramgeral[0].IDRECREVICOP;
        this.plateLocal = callback.data.dataset.paramgeral[0].IDBLOPRATLOC;

        if (result === 'NOPERMISSION' && receitaRevisada === 'S') this.utilities.showAlert('Alert', 'pme::pratoLocalOpSemPermissao', 'error');
        else {
          const currentRow = plateTree.datasource.currentRow;
          this.generateCodePlate(currentRow, row);
        }
      } else this.utilities.showAlert('Alert', 'pme::paramPlanejaSemConf', 'error');
    });
  }
  
  public generateCodePlate(currentRow: any, row: any) {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    let pratoPai = currentRow.CDPRATO === undefined ? '' : currentRow.CDPRATO;
    const filAux = currentRow.CDFILAUXPRAT === undefined ? 'P' : currentRow.CDFILAUXPRAT;
    let levelPlate = currentRow.CDNVPRATO === undefined ? 0 : currentRow.CDNVPRATO;
    const unity = row.UNIDADEFORM;
    const maskLevel = this.getLevelByMask(1);
    const nivelMax = maskLevel.length;

    if (levelPlate === nivelMax.toString()) {
      levelPlate -= 1;
      pratoPai = pratoPai.substr(0, maskLevel[levelPlate - 1].caracteres);
    }

    if ((levelPlate === 0) || (levelPlate === 1 && pratoPai === '')) {
      levelPlate = 0;
      pratoPai = 0;
    }

    const searchLevel = this.searchLevelOpen(currentRow, plateTree);
    
    if (searchLevel.length > 0) {
      pratoPai = ((searchLevel.split('_'))[0].substr(-(maskLevel[levelPlate].caracteres))).split('.').join('.');
      levelPlate = (searchLevel.split('_'))[1];
    }
    Http.post('/prato/searchNewCode', {
      pratoPai, filAux, levelPlate, unity,
    }).then((newCode) => {
      const result = newCode.data.dataset.CODIGO[0].toString();
      let code;
      if (result === 0) code = result.padEnd(maskLevel[levelPlate].caracteres, '0');
      else code = result.padStart(maskLevel[levelPlate].caracteres, '0');

      if (code) this.openModalAddPlate(code);
    });
  }

  public searchLevelOpen(currentRow: any, plateTree: TreeGridEditable) {
    let pratoPaiNivelPai = '';
    let nivelAtual = 1;
    let prato;
    
    if (currentRow.CDNVPRATO > 1) nivelAtual = currentRow.CDNVPRATO;

    plateTree.datasource.data.forEach((dado: IDictionary) => {
      if (currentRow.CDPRATO) {
        if (dado.CDPRATO.split('.').join('') === currentRow.CDPRATO.split('.').join('')) {
          prato = dado;
        }
      }
    });

    if (!prato) prato = currentRow;

    while (nivelAtual > 1) {
      const parentPlate = plateTree.datasource.currentRow.tree__parent;
      const parentPlateOpened = plateTree.datasource.currentRow.tree__parent.tree__opened;
      if (parentPlate) {
        if (!parentPlateOpened) {
          pratoPaiNivelPai = nivelAtual === 2 ? `${parentPlate.CDPRATO}_${parentPlate.CDNVPRATO}` : `${parentPlate.tree__parent.CDPRATO}_${parentPlate.tree__parent.CDNVPRATO}`;
        }
      }
      prato = parentPlate;
      nivelAtual -= 1;
    }
    return pratoPaiNivelPai;
  }

  public getLevelByMask(firstLevel: any) {
    const mask: string = (Config.masks.plateMask as PmeMask).pattern;
    const splitMask = mask.split('.');
    const biggestLevel = splitMask.length;

    const level: any = [];
    let objLevel = {};

    for (let i = 0; i < biggestLevel; i += 1) {
      objLevel = {
        nivel: i + firstLevel,
        caracteres: i > 0 ? level[i - 1].caracteres + splitMask[i].length : splitMask[i].length,
      };

      level.push(objLevel);
    }
    return level;
  }

  public reloadTree() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const unidade = Metadata.getInstance<Form>('formUnidade');
    if (unidade.validate()) {
      plateTree.buildTree();
      if (plateTree.editing) this.cancelEditingTreeGrid();
      plateTree.selectedRows = [];
    } else plateTree.datasource.data = [];
  }

  public cancelEditingTreeGrid() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    plateTree.cancelEditedRows();
    const toolbarSlot = plateTree.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = false;
      if (e.name === 'saveButton') e.disabled = false;
    });
    plateTree.buildTree();
  }

  // Funções Modal Add Plate

  private async createModalAddPlate() {
    this.modalAddPlate = await ModalService.createFromJson('modals/formAddPrato', true);
  }

  public closeModalAddPlate() {
    this.modalAddPlate.hide();
  }
  
  public openModalAddPlate(code: any) {
    this.codeTree = code;
    if (!this.modalAddPlate) {
      this.createModalAddPlate().then(() => {
        this.modalAddPlate.show();
      });
    } else {
      this.modalAddPlate.show();
    }
  }

  public disabledFields() {
    const form = Metadata.getInstance<Form>('formAddPlate');
    const formRow = form.value;
    const levelPlate = formRow.CDPRATO.split('.').length;
    const maskLevel = this.getLevelByMask(1);
    const levelMax = maskLevel.length;
    maskLevel.forEach((row: IDictionary) => {
      if (row.nivel === levelPlate) formRow.CDNVPRATO = row.nivel;
    });
    const tipo = form.children.find((column: IDictionary) => column.name === 'CDFILAUXPRAT');
    // const un = form.children.find((column: IDictionary) => column.name === 'DSUNIDADE');
    const un = form.getChildInstance<Select>('DSUNIDADE');
    const arredondTranp = form.children.find((column: IDictionary) => column.name === 'VRARREDTRANS');
    const preparo = form.children.find((column: IDictionary) => column.name === 'NMFORMAPREPARO');
    const tempo = form.children.find((column: IDictionary) => column.name === 'QTTEMPOPRAT');
    const frequencia = form.children.find((column: IDictionary) => column.name === 'IDFREQUENCIA');
    const status = form.children.find((column: IDictionary) => column.name === 'IDATIVO');
    const dateInativacao = form.children.find((column: IDictionary) => column.name === 'DTINATIVAUTO');
    
    if (tipo && un && arredondTranp && preparo && tempo && frequencia && status && dateInativacao) {
      if (formRow.CDNVPRATO === levelMax) {
        tipo.disabled = false;
        un.disabled = false;
        arredondTranp.disabled = false;
        preparo.disabled = false;
        tempo.disabled = false;
        frequencia.disabled = false;
        status.disabled = false;
        status.value = 'S';
        tipo.value = 'P';
        dateInativacao.disabled = false;
      } else if (formRow.CDNVPRATO === (levelMax - 1)) {
        tipo.disabled = false;
        tipo.value = 'P';
        un.disabled = false;
        arredondTranp.disabled = false;
        preparo.disabled = false;
        tempo.disabled = false;
        frequencia.disabled = false;
        status.disabled = false;
        status.value = 'S';
        dateInativacao.disabled = true;
      } else {
        tipo.disabled = true;
        un.disabled = true;
        un.removeValidation('required');
        arredondTranp.disabled = true;
        preparo.disabled = true;
        tempo.disabled = true;
        frequencia.disabled = true;
        status.disabled = true;
        dateInativacao.disabled = true;
        tipo.value = 'P';
        status.value = 'S';
      }
    }
  }
  
  public validTime() {
    const formModalPlate = Metadata.getInstance<Form>('formAddPlate');
    let time = formModalPlate.value.QTTEMPOPRAT;
    
    if (time < 1 && time !== null) {
      this.utilities.showAlert('alert', 'pme::valorTempoMaiorZero', 'warning');
      time = '';
    }
    
    return time;
  }
  
  public validArredond() {
    const formModalPlate = Metadata.getInstance<Form>('formAddPlate');
    let arredondamento = formModalPlate.value.VRARREDTRANS;
    if (arredondamento < 1 && arredondamento !== null) {
      this.utilities.showAlert('alert', 'pme::valorArredondMaiorZero', 'warning');
      arredondamento = '';
    }
    
    return arredondamento;
  }
  
  public savePlate() {
    const formModalPlate = Metadata.getInstance<Form>('formAddPlate');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
    const currentRow = plateTree.datasource.currentRow;
    const rowForm = formModalPlate.value;
    const maskLevel = this.getLevelByMask(1);
    const levelMax = maskLevel.length.toString();
    const permissaoPadrao = true; // grupo de acesso
    
    maskLevel.forEach((row: IDictionary) => {
      if (row.caracteres === rowForm.CDPRATO.length) rowForm.CDNVPRATO = row.nivel;
    });
    
    if (rowForm.CDNVPRATO.toString() === levelMax) {
      rowForm.QTTEMPOPRAT = this.validTime();
      rowForm.VRARREDTRANS = this.validArredond();
    }
    
    if (formModalPlate.validate()) {
      if ((rowForm.QTTEMPOPRAT >= 1 || rowForm.QTTEMPOPRAT === null) && (rowForm.VRARREDTRANS >= 1 || rowForm.VRARREDTRANS === null)) {
        if (rowForm.IDATIVO !== null) {
          if (currentRow.CDNVPRATO < levelMax - 1 && !permissaoPadrao) this.utilities.showAlert('alert', 'pme::semPermissaoAddPratoNvlComp', 'error');
          else {
            const codePlate = rowForm.CDPRATO;
            const level = formModalPlate.value.CDPRATO.split('.').length;
            let CDPRATO = '';
    
            if (level > 1) CDPRATO = codePlate.split('.').join('').substr(0, maskLevel[level - 2].caracteres);
            else CDPRATO = codePlate.split('.').join('').substr(0, maskLevel[level - 1].caracteres);
          
            Http.post('/prato/returnPlate', { CDPRATO }).then((result) => { // valid dad plate exist.
              const resultPlate = result.data.dataset.PRATO;
              if (resultPlate.CDPRATO === false && level > 1) this.utilities.showAlert('alert', 'pme::pratoPaiInexistente', 'error');
              else if (!CDPRATO && level > 1) {
                plateTree.datasource.data.forEach((row) => {
                  if (row.CDPRATO === CDPRATO) {
                    plateTree.datasource.currentRow = row;
                  }
                });
              }
              
              CDPRATO = rowForm.CDPRATO.split('.').join('');
              Http.post('/prato/returnPlate', { CDPRATO, unidade }).then((codeEqual) => { // valid plate insert exist.
                const resultCodeEqual = codeEqual.data.dataset.PRATO;
                if (resultCodeEqual.CDPRATO) this.utilities.showAlert('alert', 'pme::pratoEncontradoArvore', 'error');
                else if (this.plateLocal === 'S') {
                  if (currentRow.CDFILAUXPRAT === 'P') {
                    this.saveRowPlate(level, levelMax, rowForm);
                    // if (permissaoPadrao) this.keepSavingPrato(currentRow, rowForm, resultCodeEqual, level, levelMax); grupo de acesso
                    // else this.saveRowPlate(level, levelMax, rowForm);
                  } else this.utilities.showAlert('alert', 'pme::opBloqAddPratoLocalMarcada', 'error');
                } this.saveRowPlate(level, levelMax, rowForm);
              });
            });
          }
        } else this.utilities.showAlert('alert', 'pme::statusObrig', 'error');
      }
    }
  }
  
  public calculoValidade(tempo: any, frequencia: any) {
    if (frequencia === 'H') return tempo;
    if (frequencia === 'D') return tempo * 24;
    if (frequencia === 'M') return tempo * 24 * 30;
    if (frequencia === 'A') return tempo * 24 * 30 * 12;
    return 0;
  }
  
  public saveRowPlate(level: any, levelMax: any, rowForm: any) {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const form = Metadata.getInstance<Form>('formUnidade');
    const maskLevel = this.getLevelByMask(1);
    const buttonSave = Metadata.getInstance<Button>('savePlateAdd');
    if (level < levelMax) {
      rowForm.CDFILAUXPRAT = 'P';
      rowForm.CDFILIAL = '';
      rowForm.SGUNIDADE = '';
      rowForm.VRARREDTRANS = '';
      rowForm.NRFORMAPREPARO = '';
      rowForm.QTTEMPOPRAT = '';
      rowForm.IDFREQUENCIA = '';
      rowForm.__is_new = true;
    }
    
    rowForm.VRVALIDADE = rowForm.QTTEMPOPRAT ? this.calculoValidade(rowForm.QTTEMPOPRAT, rowForm.IDFREQUENCIA) : 0;
    rowForm.CDPRATO = rowForm.CDPRATO.split('.').join('');
    rowForm.CDFILIAL = form.value.UNIDADEFORM.CDFILIAL;
    rowForm.IDATIVO = rowForm.IDATIVO === null ? 'S' : rowForm.IDATIVO;
    maskLevel.forEach((row: IDictionary) => {
      if (row.caracteres === rowForm.CDPRATO.length) rowForm.CDNVPRATO = row.nivel;
    });
    
    this.utilities.activeLoadingButton(buttonSave);
    Http.post('/prato/savePlate', { requestType: 'Row', rowForm }).then(() => {
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      this.closeModalAddPlate();
    }).catch((error: string) => {
      if (error.includes('ORA-00001')) this.utilities.showAlert('alert', 'pme::impossivelAdicionar', 'error');
      else this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      this.utilities.disableLoadingButton(buttonSave);
      plateTree.buildTree();
    });
  }
  
  public updatePlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const editedRows = plateTree.getEditedRows();

    Http.post('/prato/updatePlate', { editedRows }).then(() => {
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      const toolbarSlot = plateTree.toolbarSlot;
      toolbarSlot.forEach((e: IComponentRender) => {
        if (e.name === 'cancelButton') e.disabled = true;
        if (e.name === 'saveButton') e.disabled = true;
      });
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      plateTree.cancelEditedRows();
      plateTree.buildTree();
    });
  }
  
  public confirmDeletePlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const editedRows = plateTree.selectedRows;
    
    if (editedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pma00200_excluiServico',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteTree(plateTree, editedRows) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public deleteTree(plateTree: TreeGridEditable, editedRows: any) {
    const buttonYes = Metadata.getInstance<Button>('yes');
    this.utilities.activeLoadingButton(buttonYes);

    Http.post('/prato/deletePlate', { editedRows }).then((response) => {
      const excludeResponse = response.data.dataset.exclude;
      if (excludeResponse.type === 'error') this.utilities.showAlert('excludePlate', excludeResponse.message, excludeResponse.type);
      else {
        this.utilities.showAlert('excludePlate', excludeResponse.message, excludeResponse.type);
        plateTree.selectedRows = [];
        plateTree.buildTree();
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      DialogService.hide();
      this.utilities.disableLoadingButton(buttonYes);
    });
  }

  // Funções da aba Embalagem

  public setValueQty({ component }: IEventParam<Select>) {
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const qtContainer = component.datasource.currentRow.QTCONTAINER;

    if (qtContainer) grid.datasource.currentRow.QTPRATCONTAINER = qtContainer;
  }

  public saveEmbalagem() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const editedRows = grid.getEditedRows();
    if (editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDPRATO = plateTree.datasource.currentRow.CDPRATO;
          row.CDFILAUXPRAT = plateTree.datasource.currentRow.CDFILAUXPRAT;
          row.CDCONTAINER = row.NMCONTAINER.CDCONTAINER;
          row.NMCONTAINER = row.NMCONTAINER.NMCONTAINER;
          row.NRORG = '';
        }
      });

      if (editedRows[0].IDPRATCONTPRI === 'S') {
        let update;
        grid.datasource.data.forEach((item: IDictionary) => {
          if (item.IDPRATCONTPRI === 'S' && item.CDCONTAINER !== editedRows[0].CDCONTAINER) {
            item.__is_new = false;
            update = item;
          }
        });

        if (update) {
          Http.post('/prato/updatePackaging', { requestType: 'Row', update }).then(() => {
            this.callSavePack(editedRows, grid);
          });
        } else this.callSavePack(editedRows, grid);
      } else this.callSavePack(editedRows, grid);
    }
  }

  public callSavePack(editedRows: any, grid: GridEditable) {
    this.crudController.saveRows(editedRows, 'pratocontainer', grid).then(() => {
      this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
      this.gridEditController.defaultState(grid);
      // this.gridEditController.reload({ grid });
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  // SetForm Dados do Prato
  
  public mountedDadosPlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formDadosPlate = Metadata.getInstance<Form>('formDadosPrato');
    const rowDadosPlate = formDadosPlate.value;
    rowDadosPlate.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
    rowDadosPlate.CDPRATO = currentRow.CDPRATO;
    rowDadosPlate.NMPRATO = currentRow.NMPRATO;
    const dsUnid = Metadata.getInstance<Select>('DSUNIDADEDP');
    const prepare = Metadata.getInstance<Select>('NMFORMAPREPARODP');
    rowDadosPlate.CDFILIAL = currentRow.CDFILIAL;
    
    Http.post('/prato/resultDadosPlate', { rowDadosPlate }).then((result) => {
      const retorno = result.data.dataset.result[0];
      rowDadosPlate.DTINATIVAUTO = retorno.DTINATIVAUTO;
      rowDadosPlate.IDATIVO = retorno.IDATIVO;
      rowDadosPlate.IDFREQUENCIA = retorno.IDFREQUENCIA;
      rowDadosPlate.QTTEMPOPRAT = retorno.QTTEMPOPRAT;
      rowDadosPlate.VRARREDTRANS = retorno.VRARREDTRANS;
      const dsUnidade = {
        SGUNIDADE: retorno.SGUNIDADE,
        DSUNIDADE: retorno.DSUNIDADE,
      };
      dsUnid.datasource.data = [dsUnidade];
      dsUnid.setValue(retorno.SGUNIDADE);
      const formaPreparo = {
        NRFORMAPREPARO: retorno.NRFORMAPREPARO,
        NMFORMAPREPARO: retorno.NMFORMAPREPARO,
      };
      prepare.datasource.data = [formaPreparo];
      prepare.setValue(retorno.NRFORMAPREPARO);
    });
  }
  
  public disableButtonsDadosPlate() {
    const cancel = Metadata.getInstance<Button>('cancelButtonDadosPlate');
    const save = Metadata.getInstance<Button>('saveButtonDadosPlate');
    cancel.disabled = false;
    save.disabled = false;
  }
  
  public cancelForm() {
    const cancel = Metadata.getInstance<Button>('cancelButtonDadosPlate');
    const save = Metadata.getInstance<Button>('saveButtonDadosPlate');
    cancel.disabled = true;
    save.disabled = true;
  }
  
  public updateDadosPlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const formDadosPlate = Metadata.getInstance<Form>('formDadosPrato');
    const rowDadosPlate = formDadosPlate.value;

    Http.post('/prato/updateDadosPlate', { rowDadosPlate }).then(() => {
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      this.cancelForm();
      plateTree.buildTree();
      // this.mountedDadosPlate();
    });
  }
  
  // Receitas
  
  public setFilterUnidadeLocal() {
    const formUnidade = Metadata.getInstance<Form>('formUnidade');
    const unidLocal = Metadata.getInstance<Select>('BRANCH_LOCAL');
    const values = {
      CDFILIAL: formUnidade.value.UNIDADEFORM.CDFILIAL,
      NMFILIAL: formUnidade.value.UNIDADEFORM.NMFILIAL,
    };
    unidLocal.datasource.data = [values];
    unidLocal.setValue(formUnidade.value.UNIDADEFORM.CDFILIAL);
  }
  
  public setFilterUnidadeService() {
    const formUnidade = Metadata.getInstance<Form>('formUnidade');
    const unidLService = Metadata.getInstance<Select>('BRANCH_SERVICE');
    const values = {
      CDFILIAL: formUnidade.value.UNIDADEFORM.CDFILIAL,
      NMFILIAL: formUnidade.value.UNIDADEFORM.NMFILIAL,
    };
    unidLService.datasource.data = [values];
    unidLService.setValue(formUnidade.value.UNIDADEFORM.CDFILIAL);
  }
  
  // Receita Padrão
  
  public setfilterRecipeStandard() {
    const grid: GridEditable = Metadata.getInstance('RECIPE_STANDARD');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT,
      },
    ];
    grid.datasource.filter = conditions;
    grid.datasource.get().then(() => {
      // this.loadStandardRecipeData(conditions);
      const addButton: Button = grid.getChildInstance('addButton');
      addButton.disabled = false;
      // this.enabledTabs();
    });
  }
  
  public saveRecipeStandard({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeStandard');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.QTLIMSEGPADR = e.QTLIMSEGPADR || null;
        e.QTCOMPLPRODPADR = e.QTCOMPLPRODPADR || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPR && !this.utilities.timeIsValid(e.HRTEMPPREPPR)) validHour = false;
        e.HRTEMPPREPPR = this.utilities.formatHour(e.HRTEMPPREPPR, false);
      });

      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeStandard', rows, grid);
    }
  }
  
  public getFilterProd() {
    const unidade = Metadata.getInstance<Select>('UNIDADEFORM');
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value.CDFILIAL,
      },
    ];
  }
  
  // Receita Local
  
  public setFilterRecipeLocal() {
    if (document.getElementsByName('RECIPE_LOCAL').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_LOCAL');
      const form = Metadata.getInstance<Form>('FORM_LOCAL');
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
  
      const conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: currentRow.CDPRATO,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: form.value.BRANCH_LOCAL.CDFILIAL,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: currentRow.CDFILAUXPRAT,
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get().then(() => {
        const addButton: Button = grid.getChildInstance('addButton');
        addButton.disabled = false;
      });
    }
  }
  
  public saveRecipeLocal({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeLocal');
    const formUnidLocal = Metadata.getInstance<Form>('FORM_LOCAL');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILIAL = formUnidLocal.value.BRANCH_LOCAL.CDFILIAL;
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.QTLIMSEGFILI = e.QTLIMSEGFILI || null;
        e.QTCOMPLPRODFILI = e.QTCOMPLPRODFILI || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPLO && !this.utilities.timeIsValid(e.HRTEMPPREPLO)) validHour = false;
        e.HRTEMPPREPLO = this.utilities.formatHour(e.HRTEMPPREPLO, false);
      });
      
      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeLocal', rows, grid);
    }
  }
  
  // Receita Serviço
  
  public setFilterRecipeService() {
    if (document.getElementsByName('RECIPE_SERVICE').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_SERVICE');
      const form = Metadata.getInstance<Form>('FORM_SERVICE');
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      const conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: currentRow.CDPRATO,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: form.value.BRANCH_SERVICE.CDFILIAL,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: form.value.SERVICE.CDSERVFILI,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: currentRow.CDFILAUXPRAT,
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get().then(() => {
        const addButton: Button = grid.getChildInstance('addButton');
        addButton.disabled = false;
      });
    }
  }
  
  public saveRecipeService({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeService');
    const formService = Metadata.getInstance<Form>('FORM_SERVICE');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILIAL = currentRow.CDFILIAL;
        e.CDSERVFILI = formService.value.SERVICE.CDSERVFILI;
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDADE;
        e.QTLIMSEGSERV = e.QTLIMSEGSERV || null;
        e.QTCOMPLPRODSERV = e.QTCOMPLPRODSERV || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPSE && !this.utilities.timeIsValid(e.HRTEMPPREPSE)) validHour = false;
        e.HRTEMPPREPSE = this.utilities.formatHour(e.HRTEMPPREPSE, false);
      });
      
      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeService', rows, grid);
    }
  }
  
  //  Receita Tipo de Cardápio
  
  public isVisibleTabTypeMenu() {
    const tabPlate = Metadata.getInstance<Tabs>('tabPratos');
    const tabTypeMenu = tabPlate.tabs.find((column: IDictionary) => column.name === 'tabReceitaTipoCardapio');
    if (tabTypeMenu) {
      if (localStorage.getItem('PRODUCT_ID') === '4875') tabTypeMenu.isVisible = true;
      else tabTypeMenu.isVisible = false;
    }
  }
  
  public setfilterRecipeTpCard() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    if (document.getElementsByName('RECIPE_TIPOCARD').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_TIPOCARD');
      const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
      
      const conditions = [
        {
          name: 'P_CDPRATO',
          operator: '=',
          value: currentRow.CDPRATO,
        },
        {
          name: 'P_CDFILAUXPRAT',
          operator: '=',
          value: currentRow.CDFILAUXPRAT,
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'P_CDTIPOCARD',
          operator: '=',
          value: formTpCard.value ? formTpCard.value.TIPOCARD.CDTIPOCARD : null,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get().then(() => {
        const addButton: Button = grid.getChildInstance('addButton');
        addButton.disabled = false;
      });
    }
  }
  
  public saveRecipeTpCard({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeTpcard');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDTIPOCARD = formTpCard.value.TIPOCARD.CDTIPOCARD;
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPRTP && !this.utilities.timeIsValid(e.HRTEMPPREPPRTP)) validHour = false;
        e.HRTEMPPREPPRTP = this.utilities.formatHour(e.HRTEMPPREPPRTP, false);
        e.CDFILCENTPME = this.appController.getCdCentral();
      });

      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeTpCard', rows, grid);
    }
  }
  
  // Receita Padrão de Qualidade e Região
  
  public visibleRecipeStandardQualityAndRegion() {
    const tabPlate = Metadata.getInstance<Tabs>('tabPratos');
    const tabStandardQuality = tabPlate.tabs.find((column: IDictionary) => column.name === 'tabReceitaPadraoQualidade');
    const tabRegion = tabPlate.tabs.find((column: IDictionary) => column.name === 'tabReceitaRegiao');
    Http.post('/prato/returnParamTco', {}).then((result) => {
      const paramTco = result.data.dataset.paramTco[0];
      if (tabStandardQuality && tabRegion) {
        if (paramTco === 'N' || paramTco.IDREVRECPADQREGIAO === 'N') {
          tabStandardQuality.isVisible = false;
          tabRegion.isVisible = false;
        } else {
          tabStandardQuality.isVisible = true;
          tabRegion.isVisible = true;
        }
      }
    });
  }
  
  public setfilterRecipeRegion() {
    if (document.getElementsByName('RECIPE_REGION').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_REGION');
      const form = Metadata.getInstance<Form>('FORM_REGIAO');
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      if (form.validate()) {
        const conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
          {
            name: 'NRPADRAOQUALIDADE',
            operator: '=',
            value: form.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE,
          },
          {
            name: 'CDREGIAO',
            operator: '=',
            value: form.value.NMREGIAO.CDREGIAO,
          },
        ];
        grid.datasource.filter = conditions;
        grid.datasource.get().then(() => {
          const addButton: Button = grid.getChildInstance('addButton');
          addButton.disabled = false;
        });
      }
    }
  }
  
  public saveRecipeRegion({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeRegion');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.NRRECEREGIAO = '';
        e.NRPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
        e.CDPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.CDPADRAOQUALIDADE;
        e.CDREGIAO = formRegiao.value.NMREGIAO.CDREGIAO;
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPRTP && !this.utilities.timeIsValid(e.HRTEMPPREPPRTP)) validHour = false;
        e.HRTEMPPREPPRTP = this.utilities.formatHour(e.HRTEMPPREPPRTP, false);
        e.CDFILCENTPME = this.appController.getCdCentral();
      });

      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeRegion', rows, grid);
    }
  }
  
  public setfilterRecipePadraoQualidade() {
    if (document.getElementsByName('RECIPE_PADRAOQUALIDADE').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_PADRAOQUALIDADE');
      const form = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      if (form.validate()) {
        const conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
          {
            name: 'NRPADRAOQUALIDADE',
            operator: '=',
            value: form.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE,
          },
        ];
        grid.datasource.filter = conditions;
        grid.datasource.get().then(() => {
          const addButton: Button = grid.getChildInstance('addButton');
          addButton.disabled = false;
        });
      }
    }
  }
  
  // Modal Importação
  
  public async createImportModal() {
    // this.receitaController.setTabPlate();
    this.modalImport = await ModalService.createFromJson('/modals/importacaoReceita', true);
  }
  
  public showModalImport({ component }: IDictionary) {
    this.modalImport.show();
    const name = component.name;
    if (name.includes('PADRAO') > 0) this.recipeDest = 'PADRAO';
    else if (name.includes('TIPOCARD') > 0) this.recipeDest = 'TIPOCARD';
    else if (name.includes('LOCAL') > 0) this.recipeDest = 'LOCAL';
    else this.recipeDest = 'SERVICO';

    // const gridDest = this.utilities.returnGrid(component) as GridEditable;
    // this.prodDest = gridDest.datasource.data;
  }
  
  public closeModalImport() {
    this.modalImport.hide();
  }
}
