import {
  Form, Grid, Text, Column,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, Loader, I18n, Http,
} from '@zeedhi/core';

export class Pla01304ConsultaCustosController {
  private utilities = Loader.getInstance('Utilities');
  
  private nrcorsafra: any = '';
  
  private nrcorinicsaf: any = '';
  
  private nrcorentrsaf: any = '';
  
  public currency: any = '';
  
  public getParamPlanejaEFilial() {
    this.utilities.getParamPlaneja('0100').then((paramplaneja: IDictionary) => {
      this.nrcorsafra = this.utilities.TColorToHex(paramplaneja?.NRCORSAFRA);
      this.nrcorinicsaf = this.utilities.TColorToHex(paramplaneja?.NRCORINICSAF);
      this.nrcorentrsaf = this.utilities.TColorToHex(paramplaneja?.NRCORENTRSAF);
    });
    this.currencySymbol();
  }
  
  private async filterPlatesGrid() {
    const filter: IDictionary = [];
    const gridConsultaCustos: Grid = Metadata.getInstance('CONSULTACUSTOS');
    const formConsultaCustos: Form = Metadata.getInstance('FORMCONSULTACUSTOS');
    const totalCustos: Text = Metadata.getInstance('CustoPerCaptaPratoResult');
    const safraResult: Text = Metadata.getInstance('SafraResult');
    const formRow = formConsultaCustos.value;
    
    let custoTotalPerCapta: any = 0;
    let percentualSafra = 0;

    setTimeout(() => {
      if (formConsultaCustos.validate()) {
        this.utilities.onMountedFilterFind(filter, 'CDFILIALCUSTO', '=', '0100');
        this.utilities.onMountedFilterFind(filter, 'NRTIPOCUSTO', '=', formRow.NMTIPOCUSTO);
        this.utilities.onMountedFilterFind(filter, 'DTCUSTO', '=', formRow.DATEINPUT);
        this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', null);
        this.utilities.onMountedFilterFind(filter, 'MES', '=', '01');
        this.utilities.onMountedFilterFind(filter, 'CDFILAUXPRAT', '=', 'P');
        this.utilities.onMountedFilterFind(filter, 'CDPRATO', '=', '0705010044');
        this.utilities.onMountedFilterFind(filter, 'CDFILIAL', '=', null);
        this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', '0001');
        this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', null);
        
        gridConsultaCustos.datasource.filter = filter;
        gridConsultaCustos.datasource.get().then(() => {
          gridConsultaCustos.datasource.data.forEach((row) => {
            custoTotalPerCapta += row.CUSTONOPRATO;
            if (row.IDINTESAFR != null) {
              if (row.IDINTESAFR === '2') percentualSafra += (((row.CUSTONOPRATO.toFixed(5) / custoTotalPerCapta.toFixed(5)) * 100) / 2);
              if (row.IDINTESAFR === '3') percentualSafra += ((row.CUSTONOPRATO.toFixed(5) / custoTotalPerCapta.toFixed(5)) * 100);
            } else row.IDINTESAFR = 0;
          });
          this.setCurrencyRow();
          totalCustos.text = custoTotalPerCapta.toFixed(5).replace('.', ',');
          safraResult.text = `${percentualSafra.toFixed(5).replace('.', ',')}${' %'}`;
          if (percentualSafra > 0) {
            Http.post('/prato/returnPercentSafra', { requestType: 'Row', percentualSafra }).then((returnSafra) => {
              const returnColorSafra = returnSafra.data.dataset.result[0].NRCORSINSAFR;
              const colorSafra = this.utilities.TColorToHex(returnColorSafra.toString());
              document.getElementsByName('SafraResult')[0].style.color = colorSafra;
            });
          }
        });
      } else {
        gridConsultaCustos.datasource.data = [];
        totalCustos.text = '0';
        safraResult.text = '0%';
      }
    }, 2);
  }
  
  public returnColumn(component: Grid, nmColuna: string) {
    return component.columns.find((column) => column.name === nmColuna);
  }
  
  public setCurrencyRow() {
    const gridConsultaCustos: Grid = Metadata.getInstance('CONSULTACUSTOS');
    const custo = this.returnColumn(gridConsultaCustos, 'CUSTO') as Column;
    const custoPrato = this.returnColumn(gridConsultaCustos, 'CUSTONOPRATO') as Column;
    const currency = this.currency;
    
    custo.componentProps.mask.currencySymbol = currency;
    custoPrato.componentProps.mask.currencySymbol = currency;
  }
  
  public currencySymbol() {
    const currency = localStorage.getItem('CURRENCY');
    this.currency = currency;
  }
  
  private checkLanguage() {
    const language = this.utilities.getLanguage() as string;
    let idioma = null;
    if (language != null && language.includes('en')) idioma = 'I';
    else if (language != null && language.includes('es')) idioma = 'E';
    return idioma;
  }
  
  private getSafColor({ row }: IDictionary) {
    let decColor = 0;
    const nrcorsafra = this.utilities.hexToRgbInline(this.nrcorsafra).split('.', 3);
    const nrcorinicsaf = this.utilities.hexToRgbInline(this.nrcorinicsaf).split('.', 3);
    const nrcorentrsaf = this.utilities.hexToRgbInline(this.nrcorentrsaf).split('.', 3);
    
    if (row.IDINTESAFR > 0) {
      if (row.IDINTESAFR === '1') {
        decColor = this.utilities.RGBToTColor(nrcorsafra[0], nrcorsafra[1], nrcorsafra[2]);
        row.SAFRA = I18n.translate('pme::safra');
      }
      if (row.IDINTESAFR === '2') {
        decColor = this.utilities.RGBToTColor(nrcorinicsaf[0], nrcorinicsaf[1], nrcorinicsaf[2]);
        row.SAFRA = I18n.translate('pme::inicioFimSafra');
      }
      if (row.IDINTESAFR === '3') {
        decColor = this.utilities.RGBToTColor(nrcorentrsaf[0], nrcorentrsaf[1], nrcorentrsaf[2]);
        row.SAFRA = I18n.translate('pme::entreSafra');
      }
    } else row.IDINTESAFR = '';
    
    const color = this.utilities.TColorToHex(decColor.toString());
    
    return color;
  }
}
