import {
  Modal, ModalService, Radio,
  CheckboxMultiple, SelectMultiple, Form,
  Button, DialogService,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00600ExclusaoPrato {
  private modalExclusao!: Modal;
    
  public showSelectSpecific = false;
    
  public utilities = Loader.getInstance('Utilities');
    
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
    
  public async createModal() {
    this.modalExclusao = await ModalService.createFromJson('modals/pme00600_exclusao_prato', true);
  }
    
  public cancelModal() {
    if (this.modalExclusao) {
      this.modalExclusao.hide();
      this.showSelectSpecific = false;
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(false);
    }
  }
    
  public showModal({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    this.validAndOpenModal();
  }
    
  public async validAndOpenModal() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    if (cardapio.filterData.NRCARDPME !== undefined) {
      cardapio.setValueAction(true);
      if (this.modalExclusao) {
        this.modalExclusao.show();
      } else {
        this.createModal().then(() => {
          this.modalExclusao.show();
        });
      }
    } else {
      this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
    }
  }
    
  public setInitialDate({ component }: IEventParam<IDictionary>) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const date = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
    if (component.name === 'initialDateExclude') {
      component.value = date;
    } else {
      component.value = date;
    }
  }
    
  public changeVisibilitySelect({ component }: IEventParam<Radio>) {
    if (component.value === 'E') this.showSelectSpecific = true;
    else this.showSelectSpecific = false;
  }
    
  public showPlates({ component }: IEventParam<CheckboxMultiple>) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    let newArray = JSON.parse(JSON.stringify(cardapio.cardData)); // Copia os dados retirando a referencia do objeto.
        
    newArray = this.modifyObjectCheckBox(newArray);
    newArray = this.selectAllPlate(newArray);
    (component.datasource as IDictionary).data = newArray;
  }
    
  // Marca todos os pratos do checkbox...
  private selectAllPlate(uniqueData: IDictionary) {
    const boxPlate = Metadata.getInstance<CheckboxMultiple>('checkboxPlates');
    uniqueData.forEach((data: IDictionary) => boxPlate.value.push(data.CDPRATSEQ));
    return uniqueData;
  }
    
  // Faz a junção do tipo de prato e o sequencial, para a identificar quem é quem....
  private modifyObjectCheckBox(uniqueData: IDictionary) {
    uniqueData.forEach((data: IDictionary) => {
      data.CDPRATSEQ = `${data.CDTIPOPRAT} - ${data.NRSEQCNFCPME}`;
    });
    return uniqueData;
  }
    
  public filterSelectMultiple({ component }: IEventParam<SelectMultiple>) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cardapio.filterData.cdfilcentpme,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cardapio.filterData.typeMenu.CDTIPOCARD,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cardapio.filterData.service.CDSERVFILI,
      },
    ];
      
    component.datasource.filter = filter;
    component.datasource.get();
  }
    
  public showMessageExclude() {
    const plates = Metadata.getInstance<CheckboxMultiple>('checkboxPlates').value;
    const form = Metadata.getInstance<Form>('formExclude');
    if (form.validate()) {
      if (plates.length > 0) {
        DialogService.show({
          name: 'excludePeriodoCardapio',
          type: 'warning',
          title: 'pme::atencao',
          text: 'pme::desejaExcluirPeriodoCardapio',
          buttons: [
            {
              name: 'progressLine',
              component: 'ZdProgress',
              label: 'Progress',
              indeterminate: true,
              cssClass: 'loading-progress-line',
            },
            {
              name: 'yesButtonExclude',
              component: 'ZdButton',
              label: 'pme::sim',
              events: { click: () => this.exclude() },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'pme::nao',
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      } else this.utilities.showAlert('selectTypePlate', 'pme::nenhumTipoDePratoSelecionado', 'error');
    }
  }
    
  public exclude() {
    const form: Form = Metadata.getInstance('formExclude');
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
      
    if (form.validate()) {
      const initialDate = cardapio.dates.find((date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.initialDateExclude);
      const finalDate = cardapio.dates.find((date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.finalDateExclude);

      if (initialDate !== undefined && finalDate !== undefined) {
        if (!this.bloqDayInPeriod(form.value.initialDateExclude, form.value.finalDateExclude, cardapio)) {
          const row = form.value;
          const buttonLoad: Button = Metadata.getInstance('yesButtonExclude');
          row.platesSelect = Metadata.getInstance<CheckboxMultiple>('checkboxPlates').value;
          row.filterData = cardapio.filterData;
          this.utilities.activeLoadingButton(buttonLoad);
          this.callExcludeRoute(row).then((excludeResponse: IDictionary) => {
            if (excludeResponse.type === 'success') {
              const elaboracao = Loader.getInstance('Pme00600ElaboracaoCardapio');
              elaboracao.reloadCard();
              this.cancelModal();
              const acompanhamentoCusto = Loader.getInstance('Pla00106AcompanhamentoCusto');
              acompanhamentoCusto.autoUpdateAcomp(initialDate, finalDate);
            }
            this.utilities.showAlert('excludePlate', excludeResponse.message, excludeResponse.type);
          }).finally(() => {
            this.utilities.disableLoadingButton(buttonLoad);
            DialogService.hide();
          });
        } else {
          this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
          DialogService.hide();
        }
      } else {
        this.utilities.showAlert('outOfPeriod', 'pme::dataForaDoPeriodo', 'error');
        DialogService.hide();
      }
    }
  }
    
  private bloqDayInPeriod(initialDate: string, finalDate: string, cardapio: Cardapio): boolean {
    let hasBloqDay = false;
    cardapio.dates.forEach((date: IDictionary) => {
      if (date.DTCARDIAPME.substring(0, 10) >= initialDate && date.DTCARDIAPME.substring(0, 10) <= finalDate) {
        if (!cardapio.dateLessDateOfProcess(date.DTCARDIAPME.substring(0, 10))) hasBloqDay = true;
      }
    });
    return hasBloqDay;
  }
    
  public callExcludeRoute(row: IDictionary) {
    return Http.post('/excludePlates', { requestType: 'Row', row }).then((response) => {
      const excludeResponse = response.data.dataset.exclude;
      return excludeResponse;
    });
  }
    
  private autoUpdateAcomp() {
    const acompanhamentoCusto = Loader.getInstance('Pla00106AcompanhamentoCusto');
    if (acompanhamentoCusto.showWindow && acompanhamentoCusto.autoUpdate) {
      const form: Form = Metadata.getInstance('formExclude');
      acompanhamentoCusto.callProcedureConsPlan(form.value.initialDateExclude, form.value.finalDateExclude);
    }
  }
    
  public defaultRadio({ component }: IEventParam<Radio>) {
    component.value = (component.datasource as IDictionary).data[0].value;
  }
}
